--
-- DragDropSetUp
--

-- this class handles the round randomization of the drag and drop game data

-- runtime initialization of the game draggableList:
-- by this point both targetList and draggableList will have been initialized.

-- the draggable list is a list of drag records, initially formatted as follows:
-- spriteNum:[#coverNum:the memberNum of sprite spr, #coverLib: the castLibNum of sprite spr, #loc:(the loc of sprite spr), #identifier:value ("#" & the name of the init frame member at this sprite), #myNum:0, #myLib:answerCast1, #matchSpriteList:0, #showflag:0 ]


-- the initial targetList is as follows:  
-- spriteNum:[#loc:(the loc of sprite spr), #identifier:value ("#" & the name of member the memberNum of sprite spr of castLib the castLibNum of sprite spr), #myNum:the memberNum of sprite spr, #myLib:the castLibNum of sprite spr, #showFlag:0]



property ancestor

property randomFlag  -- will the sprites be randomized from the pool or left as the sprite layout?
property poolOrderFlag -- set up the order of play to match the draggable pool order.

on new me
  -- set up all constants first:
  
  set ancestor = new (script "DraggableListMgmt")
  
  set randomFlag = TRUE
  set poolOrderFlag = FALSE
  
  return me
end


on destruct me
  if objectP (ancestor) then destruct (ancestor)
  set ancestor = 0
end


-- manually turn randomization off.
-- this must be done before initialize round to have any effect.

on randomOff me
  set randomFlag = FALSE
end


on setByPoolOrder me
  set randomFlag = FALSE
  set poolOrderFlag = TRUE
end


-- the deal here is to evenly distribute the draggables into the draggable sprites.
-- each target (target) should contain a roughly even number of matching draggables in the end list, even if there are more 
-- draggables than slots for draggables.

on initializeRound me  
  if randomFlag then initializeRandom (me)
  else initializeNonRandom (me)
  
  initHilitePool (me, gettargetSprites (me))
end



on initializeRandom me
  -- there is only one list of draggable members at this point
  -- in the form: [castLibNum:[#memberName:memberNum, ... ]]
  -- get that castNumber for future reference:
  set memberList = the memberList of ancestor
  set mCastNum = getPropAt (memberList, 1)
  
  -- get the temporary target/member pool:
  -- in the form: [targetSprite:[memberList], targetSprite:[memberList], ... ]
  set targetMemberLst = setUptargetPool (me)
  
  set targetNames = gettargetSpritePool (me)
  set targetList = the targetList of ancestor
  set numtargets = count (targetList)
  set currtarget = random (numtargets)
  
  set tmpLst = duplicate (the draggableList of ancestor)
  set draggableList = [:]
  
  repeat while count (tmpLst)
    -- gather draggable sprite information.
    -- randomly choose a draggable rec:
    set dragSpr = getPropAt (tmpLst, random (count (tmpLst)))
    set rec = getAProp (tmpLst, dragSpr)
    
    -- get current target information:
    set cSpr = getPropAt (targetList, currtarget)
    set cRec = getAProp (targetList, cSpr)
    set id = getAProp (cRec, #identifier)
    
    set cPool = getProp (targetMemberLst, id)
    
    if not listP (cPool) then 
      nothing
    else if count (cPool) then   -- i.e. there are members left that match the current target.
      setAProp (rec, #matchSpriteList, getAProp (targetNames, id))
      setAProp (rec, #identifier, id)
      
      set currtargetPool = getProp (targetMemberLst, id)
      set tmp = random (count (cPool))
      set mNum = getAt (cPool, tmp)
      deleteAt (cPool, tmp)
      setAProp (targetMemberLst, id, cPool)
      
      setAProp (rec, #myNum, mNum)
      setAProp (rec, #myLib, mCastNum)
      
      -- return the updated record to the draggableList:
      setAProp (draggableList, dragSpr, rec)
      
      -- delete the prop from the temporary list:
      deleteProp (tmpLst, dragSpr)
    else
      put "Integrator: You may have more draggable sprites than draggable cast members. Check it out."
    end if
    
    set currtarget = currtarget + 1
    if currtarget > numtargets then set currtarget = 1
  end repeat
  
  set the draggableList of ancestor = draggableList
end


-- the non-randomized version of initializeRound does not pull draggables from a pool.
-- Instead, it takes the current choices and sprite order of draggables and matches them to their respective targets.

on initializeNonRandom me
  -- there is only one list of draggable members at this point
  -- in the form: [castLibNum:[#memberName:memberNum, ... ]]
  -- get that castNumber for future reference:
  set memberList = the memberList of ancestor
  set mCastNum = getPropAt (memberList, 1)
  set draggablePool = getAProp (memberList, mCastNum)
  
  set targetNames = gettargetSpritePool (me)  -- [identifier:[spr,spr,spr],...]
  set targetCount = count (targetNames)
  
  set tmpLst = duplicate (the draggableList of ancestor)
  set draggableList = [:]
  
  set dCount = count (tmpLst)
  repeat with i = 1 to dCount  -- go through the draggables one by one.
    -- gather draggable sprite information.
    set dragSpr = getPropAt (tmpLst, i)
    set rec = getAProp (tmpLst, dragSpr)
    
    if poolOrderFlag then -- go by the order of the draggables pool
      set id = getPropAt (draggablePool, i)
      setAProp (rec, #identifier, id)
      setAProp (rec, #myNum, getAProp (draggablePool, id))
      setAProp (rec, #myLib, mCastNum)
    else  -- go by the order of the placed sprites.
      set id = getAProp (rec, #identifier)
      setAProp (rec, #myNum, getAProp (rec, #coverNum))
      setAProp (rec, #myLib, getAProp (rec, #coverLib))
    end if
    
    -- get and set current target information:
    set currTargetList = []
    repeat with j = 1 to targetCount
      if getPropAt (targetNames, j) = id then
        set currTargetList = getAt (targetNames, j)
        exit repeat
      end if
    end repeat
    
    setAProp (rec, #matchSpriteList, currTargetList)  
    
    -- return the updated record to the draggableList:
    setAProp (draggableList, dragSpr, rec)
    
  end repeat
  
  set the draggableList of ancestor = draggableList
end



